// Copyright 2010 Simon Massey. All rights reserved. net.sf.chex4j is licensed under the terms of the Eclipse Public License v1.0, as defined by the attached file LICENSE-ECLIPSE.txt.net.sf.chex4j is licensed under the terms of the Eclipse Public License v1.0, as defined by the attached file LICENSE-ECLIPSE.txt.
package net.sf.chex4j;

import java.io.File;
import java.io.IOException;
import java.io.PrintStream;
import java.util.HashSet;
import java.util.Set;

import javassist.CannotCompileException;
import javassist.NotFoundException;
import net.sf.chex4j.internal.ChexBase;
import net.sf.chex4j.internal.ClassFileScanner;
import net.sf.chex4j.internal.OfflineClassTransformer;

/**
 * This class is the entry point for an offline class transformer which can be 
 * used to instrument classes at build time. 
 * 
 * @author simon
 * 
 */
public class ChexOfflineMain extends ChexBase {

	public static final int SUCCESS = 0;
	public static final int BAD_ARGUMENTS = 1;
	public static final int ERROR = 2;
	
	/*package*/ static PrintStream err = System.err; // allow for redirection of output
	
	public interface ExitManager{
		public void exit(int exitCode);
	}
	
	/*package*/ static ExitManager exitManager = new ExitManager() {
		public void exit(int exitCode) {
			System.exit(exitCode);
		}
	};
	
	private OfflineClassTransformer classFileLoader = null;

	public ChexOfflineMain(Arguments arguments, String classPoolClasspath) throws NotFoundException {
		super(arguments);
		classFileLoader = new OfflineClassTransformer(arguments.ouputPath, classPoolClasspath);
	}

	public static void main( String[] args ) {
		if( args.length < 3 ){
			usage();
			exitManager.exit(BAD_ARGUMENTS);
		} else {
			String packages = args[0];
			String inputDir = args[1];
			String outputDir = args[2];
			
			StringBuilder classPoolClasspath = new StringBuilder(inputDir);
			for( int index = 3; index < args.length; index++ ){
				classPoolClasspath.append(java.io.File.pathSeparatorChar);
				classPoolClasspath.append(args[index]);
			}
			
			try {
				main(packages, inputDir, outputDir, classPoolClasspath.toString());
			} catch (Exception e) {
				e.printStackTrace();
				exitManager.exit(ERROR);
			}
		}
	}

	static /*package*/ void main(String packages, String inputDir, String outputDir, String classPoolClasspath)
			throws NotFoundException, IOException, ClassNotFoundException, CannotCompileException {
		Arguments arguments = parseArguments(packages+",dir="+outputDir);
		ChexOfflineMain chexOfflineMain = new ChexOfflineMain(arguments, classPoolClasspath);
		chexOfflineMain.scanAndTransform(inputDir);
	}

	public void scanAndTransform(String inputDir) throws IOException, NotFoundException, ClassNotFoundException, CannotCompileException {
		
		ClassFileScanner classFileScanner = new ClassFileScanner();
		
		final Set<String> classCanonicalNames = new HashSet<String>(); 
		
		classFileScanner.scanAndMatch(super.assertionsPackageSpec, new ClassFileScanner.Callback() {
			public void doInMatch(String classFile) {
				if( !classCanonicalNames.add(classFile) ) {
					err.println("WARNING WARNING classFile located more than once: "+classFile);
				}
			}
		}, new File(inputDir));
		
		for( String classCanonicalName : classCanonicalNames ){
			this.classFileLoader.loadClass(classCanonicalName);
		}

		return;
	}

	public static void usage() {
		err.println("Usage: "+ChexOfflineMain.class.getName()+" packages inputDir outputDir classpathItemA classpathItemB classpathItemC ...");
		err.println("Example: "+ChexOfflineMain.class.getName()+" net.sf.chex4j...,net.sf.chex4j.test... target/classes target/instrumentedClasses x.jar y.jar z.jar");
	}
}
