// Copyright 2010 Simon Massey. All rights reserved. net.sf.chex4j is licensed under the terms of the Eclipse Public License v1.0, as defined by the attached file LICENSE-ECLIPSE.txt.net.sf.chex4j is licensed under the terms of the Eclipse Public License v1.0, as defined by the attached file LICENSE-ECLIPSE.txt.
package net.sf.chex4j.internal;

import java.io.IOException;
import java.util.HashSet;
import java.util.Set;

import javassist.CannotCompileException;
import javassist.ClassPool;
import javassist.CtClass;
import javassist.NotFoundException;

/**
 * This class is the main instrumentation work horse. Subclasses must 
 * specialise where the classes are loaded from. 
 * @author simon
 */
abstract /*package*/ class AbstractClassTransformer {

	protected final Set<ClassInstrumentor> classInstrumentors;
	protected final ClassPool classPool;

	/*package*/ AbstractClassTransformer() {
		this.classPool = ClassPool.getDefault();
		ClassPool.releaseUnmodifiedClassFile = true;
		this.classInstrumentors = new HashSet<ClassInstrumentor>(3);
		this.classInstrumentors.add(new ConstructorInstrumentor());
		this.classInstrumentors.add(new ClassMethodsInstrumentor());
		this.classInstrumentors.add(new InterfaceMethodsInstrumentor());
	}

	public byte[] transformClassfile(String canonicalName) throws NotFoundException,
			ClassNotFoundException, CannotCompileException, IOException {

		boolean workDone = false;

		CtClass ctClass = classPool.get(canonicalName);
		if (!ctClass.isInterface() && !ctClass.isFrozen() ) {
			for (ClassInstrumentor classInstrumentor : this.classInstrumentors) {
				workDone |= classInstrumentor.instrumentClass(ctClass);
			}
		}
		byte[] returned = null;
		if (workDone) {
			returned = ctClass.toBytecode();
		}
		return returned;
	}

}