// Copyright 2010 Simon Massey. All rights reserved. net.sf.chex4j is licensed under the terms of the Eclipse Public License v1.0, as defined by the attached file LICENSE-ECLIPSE.txt.net.sf.chex4j is licensed under the terms of the Eclipse Public License v1.0, as defined by the attached file LICENSE-ECLIPSE.txt.
package net.sf.chex4j.internal;

import java.io.IOException;
import java.lang.instrument.IllegalClassFormatException;
import java.security.ProtectionDomain;
import java.util.HashSet;
import java.util.Set;
import java.util.StringTokenizer;

import javassist.ByteArrayClassPath;
import javassist.CannotCompileException;
import javassist.NotFoundException;

/**
 * This class acts as both an adapter and a decorator to an underlying 
 * ClassTransformer. It adds output to disk capability and also adapts 
 * method signatures to the JDK method signature. 
 * 
 * @author simon
 */
public abstract class ChexBase extends AbstractClassTransformer {

	protected static class Arguments {
		public Arguments(){}
		public Set<String> packages;
		public String ouputPath;
	}

	protected static Arguments parseArguments(String agentArguments) {
		Arguments arguments = new Arguments();
		Set<String> packages = new HashSet<String>();
		StringTokenizer tokenizer = new StringTokenizer(agentArguments, ",",
				false);
		while (tokenizer.hasMoreTokens()) {
			String token = tokenizer.nextToken();
			if (token.startsWith("dir=")) {
				arguments.ouputPath = token.substring(token.indexOf('=') + 1);
				continue;
			} else if (!token.endsWith("...")) {
				java.lang.System.err
						.println("WARNING WARNING chex4j javaagent: packge list entry should end with \"...\": "
								+ token);
			}
			packages.add(token);
		}
		arguments.packages = packages;
		return arguments;
	}

	protected ChexBase(Arguments arguments) {
		init(arguments);
	}

	protected void init(Arguments arguments) {
		if (arguments.ouputPath != null) {
			listeners.add(new TransformedClassWriteToDiskListener(arguments.ouputPath));
		}
		this.assertionsPackageSpec = arguments.packages;
	}
	
	private Set<TransformedClassListener> listeners = new HashSet<TransformedClassListener>(
			0); 
	protected Set<String> assertionsPackageSpec;
	
	public byte[] transformClassfile(byte[] classfileBuffer,
			String canonicalName) throws NotFoundException,
			ClassNotFoundException, CannotCompileException, IOException {
		classPool.insertClassPath(new ByteArrayClassPath(canonicalName, classfileBuffer));
		byte[] bytes = super.transformClassfile(canonicalName);
		return bytes;
	}

	/*package*/ boolean packageNameMatched(String canonicalName) {
		boolean match = false;
		for( String packageArgument : this.assertionsPackageSpec ){
			if( packageArgument.endsWith("...")){
				packageArgument = packageArgument.substring(0, packageArgument.length()-2);
				if( canonicalName.startsWith(packageArgument)){
					match = true;
					break;
				}
			}
		}
		return match;
	}

	/**
	 * This method is called by the JVM classloader and allows us to return 
	 * a new class file as a byte[] after manipulating the methods to 
	 * inject the @Pre, @Post and @Invar code.   
	 */
	public byte[] transform(ClassLoader loader, String classFileName,
			Class<?> classBeingRedefined, ProtectionDomain protectionDomain,
			byte[] classfileBuffer) throws IllegalClassFormatException {
		return transformClassByteCode(classFileName, classfileBuffer);
	}

	/* (non-Javadoc)
	 * @see net.sf.chex4j.internal.ClassByteCodeTransformer#transformClassByteCode(java.lang.String, byte[])
	 */
	public byte[] transformClassByteCode(String classFileName, byte[] classByteCode) {
		byte[] newClassByteCode = null;
		String canonicalName = classFileName.replace('/', '.');
		if( packageNameMatched(canonicalName) ){
			try {
				newClassByteCode = transformClassfile(classByteCode, canonicalName);
				if( null != newClassByteCode ){
					for( TransformedClassListener l : this.listeners){
						l.onClassTransformed(classFileName, newClassByteCode);
					}
				}
			} catch (Throwable t) {
				if (t instanceof CannotCompileException) {
					java.lang.System.err
							.println("WARNING WARNING: Cannot instrument class "
									+ classFileName
									+ " due to bad annotation code. This javaagent has not instrumented the class in question.");
				}
				t.printStackTrace();
			}
		}
		return newClassByteCode;
	}
}